/*
 * ihcp_var.h 
 * structure and "software" macro definition file for Linux 2.X driver for PCI hardcopy boards
 *
 * Tahoma Technology
 * (formerly Ikon Corporation)
 * 107 2nd Avenue North
 * Seattle, WA, USA 98109
 *
 * 206.728.6465
 * http://www.tahomatech.com
 * tahoma@tahomatech.com
 * 
 * 
 * 5 January, 1999      start of development - initial target Linux 2.0.35 (R.H. 5.1)
 * 
 * 16 March, 1999       added minor_node_name array to allow registering dev node name
 *                      with interrupt, rather than driver name
 * 
 * 2 Sept, 1999         re-code for 2.2
 *                      this version no longer compatible with earlier linux revs
 * 
 * 13 March, 2000       started working w/2.3.x
 *                      changed wait_queue type
 * 
 * 24 March, 2000       incorporated kiobuf mechanism for dma from user buffers
 *                      replaced copy buf w iopb  list to do sg dma from user buf
 *                      added pci_dev_int_level to distinguist between the value
 *                      read from the board (to use in restore after eeprom reload)
 *                      and the (possibly) mapped version from pci_dev
 *                      incorporated modified version of Kaz Kylheku's mutex stuff
 *                      for SMP
 * 
 *                      requires 2.3.x kernel
 *
 * 6 May, 2002		moved conditional variable and mutex struct defines
 *			here from ikon_mutex.h
 *
 * 28 June, 2002	deleted * box disclaimer
 *
 * 1 August, 2002	added pointer to (possible) copy buffer
 *
 * 			moved "software" macros from ihcp_reg.h to this file
 *
 *			modified IHCP_SG_LINK macro to generate .page correctly
 * 			in HIGHMEM kernel case (not via page_address())
 *
 * 			detect highmem kernel && old scatterlist, and enable
 *			copy buffer pages
 *
 * 			got rid of MODINFBANNER, updated MODULE_DESCRIPTION date
 *
 * 30 March, 2005	minor updates for compatibility with x86_64
 * 			changed PUTL, GETL macros since base ptrs are now void*
 * 			were uint32_t*
 *
 * 24 March, 2006	just update the date to match driver & ioctl mods
 */


/*
 * conditional variable and mutex typedefs
 */

typedef struct {
	spinlock_t spin;
	long flags;
} mutex_t;

typedef struct {
	wait_queue_head_t queue;
} cond_t;

enum { COND_WAIT_SUCCESS, COND_WAIT_SIGNAL, COND_WAIT_TIMEOUT };


/*
 * unit structure for boards
 */

struct ihcp_unit_t {
	u_int instance;			/* equivalent to minor number & board number    */
	mutex_t mutex;			/* used w/cond var to sleep (SMP compatibility) */
	cond_t cond;			/* cond var used w/mutex                        */
	volatile int sleeping;		/* 1 if sleeping (helps detect timeout)         */
	struct pci_dev *pci_dev_p;	/* handle for this board's pci dev structure    */
	void *plx_page_base_p;		/* virtual address of plx reg page base         */
	void *ihcp_page_base_p;		/* virtual address of ihcp reg page base        */
	void *plx_base_p;		/* virtual address of plx regs                  */
	void *ihcp_base_p;		/* virtual address of ihcp regs                 */
	void *iopb_base_p;		/* virtual add of start of io parameter blocks  */
	struct scatterlist *sg_list_p;	/* points to scatterlist (for DMA mapping)      */
	dma_addr_t dma_handle;		/* contains bus add of start of iopbs           */
	struct kiobuf *kiobuf_p;	/* ptr to kiobuf for this instance              */
	char *dma_copy_buf_p;		/* ptr to copy buffer, or NULL			*/
	volatile u_int unit_open;	/* 1 = open                                     */
	volatile u_int unit_attached;	/* 1 = attached                                 */
	u_int mode;			/* copy of print/plot mode                      */
	u_int data_stream_mode;		/* copy of data streaming mode                  */
	u_int byte_order;		/* big or little endian data flag               */
	u_long unit_flags;		/* our flag bits here                           */
	u_long dma_time;		/* dma timeout # in ticks                       */
	u_long fifo_time;		/* empty and <1/2 full #                        */
	u_long dma_time_def;		/* per unit timeout default                     */
	u_long fifo_time_def;		/* per unit timeout default                     */
	u_long vers_speed_def;		/* per unit versatec speed default              */
	u_long cent_speed_def;		/* per unit centronics speed default            */
	u_long mode_def;		/* per unit mode default                        */
	uint32_t dev_and_vendor_id;	/* device and vendor id from config regs        */
	u_long revision_id;		/* revision id from config regs                 */
	u_char int_level;		/* int level read from config register          */
	u_char pci_dev_int_level;	/* (possibly) mapped int level                  */
	char minor_node_name[16];	/* will contain this board's node name          */
};

/*
 * "software" macro definitions
 */

/*
 * this code was originally placed in the public domain
 * in order to satisfy the "tainted" test, we will declare ourselved GPL'd
 * we certainly meet the spirit of the GPL, since our source is always
 * freely available and included with the driver
 */

#ifdef	MODULE_DESCRIPTION
MODULE_DESCRIPTION("Tahoma Technology (formerly Ikon Corporation) Pci Hardcopy Driver - 2006.03.24");
#endif

#ifdef	MODULE_AUTHOR
MODULE_AUTHOR("Tahoma Technology (formerly Ikon Corporation) http://www.tahomatech.com");
#endif

#ifdef	MODULE_LICENSE
MODULE_LICENSE("GPL");
#endif

/*
 * version code macro
 */

#define	VERSION_CODE(vers, rel, seq) (((vers)<<16 | ((rel)<<8) | seq))

/*
 * pick up the uts release (should be in version.h)
 */

#ifndef UTS_RELEASE
#define		UTS_RELEASE	"unknown"
#endif

/*
 * debug macro - expands to nothing unless IHCP_DEBUG is defined
 * 
 * requires double quoted arg list - printk can take
 * variable # of arguments
 */

#ifdef	IHCP_DEBUG
#define	DPRINT(args)	printk args
#else
#define	DPRINT(args)
#endif

/*
 * define and select alloc|free_kiovec macros
 * RH7.2 (updated, not base release) uses _sz functions
 * nobody else does - but we will try to accomodate
 *
 * third arg is buffer head count - we don't use - set to 0
 */

#ifdef	IHCP_RH72PLUS
#define		IHCP_ZERO int ihcp_zero = 0
#define		IHCP_ALLOC_KIOVEC(count,pointer) \
			alloc_kiovec_sz(count,pointer,&ihcp_zero)

#define		IHCP_FREE_KIOVEC(count,pointer) \
			free_kiovec_sz(count,pointer,&ihcp_zero)
#else
#define		IHCP_ZERO
#define		IHCP_ALLOC_KIOVEC(count,pointer) alloc_kiovec(count,pointer)
#define		IHCP_FREE_KIOVEC(count,pointer) free_kiovec(count,pointer)
#endif

/*
 * detect HIGHMEM enabled kernel and IHCP_HIGHMEM mode
 * if HIGHMEM machine/kernel, and only old style scatterlist available,
 * alow creation of a kernel copy buffer
 */

#if	!defined(IHCP_HIGHMEM) && !defined(CONFIG_HIGHMEM)
#define		IHCP_COPY_BUF	0
#define		IHCP_MEMORY_MODE_MSG	"IHCP_HIGHMEM=NO, CONFIG_HIGHMEM=NO, user DMA enabled\n"
#endif
#if	!defined(IHCP_HIGHMEM) && defined(CONFIG_HIGHMEM)
#define		IHCP_COPY_BUF	1
#define		IHCP_MEMORY_MODE_MSG	"IHCP_HIGHMEM=NO, CONFIG_HIGHMEM=YES, copy buffer enabled\n"
#endif
#if	defined(IHCP_HIGHMEM) && !defined(CONFIG_HIGHMEM)
#define		IHCP_COPY_BUF	0
#define		IHCP_MEMORY_MODE_MSG	"IHCP_HIGHMEM=YES, CONFIG_HIGHMEM=NO, user DMA enabled\n"
#endif
#if	defined(IHCP_HIGHMEM) && defined(CONFIG_HIGHMEM)
#define		IHCP_COPY_BUF	0
#define		IHCP_MEMORY_MODE_MSG	"IHCP_HIGHMEM=YES, CONFIG_HIGHMEM=YES, user DMA enabled\n"
#endif

/*
 * define and select macro to set the address portion of the DMA scatterlist
 * old style used .address, new style uses .page and .offset
 * later 2.4 and probably 2.5 uses .page style
 * new style should also work w/machines w/<4G ram
 */

#ifdef	IHCP_HIGHMEM
#define		IHCP_SG_LINK(sg_link,page_ptr,page_offset,page_len) \
		do { \
			sg_link.page = page_ptr; \
			sg_link.offset = page_offset; \
			sg_link.length = page_len; \
\
DPRINT((KERN_DEBUG "ihcp_strategy: instance %d: (new style sg link) page_ptr = 0x%p \
page_offset = 0x%x, page_len = 0x%x\n", instance, sg_link.page,\
sg_link.offset, sg_link.length));\
\
		} while(0)
#else
#define		IHCP_SG_LINK(sg_link,page_ptr,page_offset,page_len) \
		do { \
			sg_link.address = (char *)page_address(page_ptr) + page_offset; \
			sg_link.length = page_len; \
\
DPRINT((KERN_DEBUG "ihcp_strategy: instance %d: (old style sg link) \
address = 0x%p, length = 0x%x\n", instance,\
sg_link.address, sg_link.length));\
\
		} while(0)
#endif

/*
 * we are 32 bit DMA capable
 */

#define	IHCP_DMA_MASK	0xffffffff

/*
 * use dma_mask if required -- this is probably redundant, since we
 * are 32 bit DMA capable, and that is supposed to be the default
 */

#ifdef	IHCP_USE_DMA_MASK
#define		IHCP_PCI_SET_DMA_MASK(pci_dev_p,dma_mask) pci_set_dma_mask(pci_dev_p,dma_mask)
#else
#define		IHCP_PCI_SET_DMA_MASK(pci_dev_p,dma_mask) 0
#endif

/*
 * macro definitions for memory mapped board register access
 * and io control block memory access
 * 
 * these are gathered here to try to allow for easy portability to
 * arch's other than i386, where access to Pci board registers, and
 * the board's access to DMA chain list memory may be byte swizzled
 * 
 * the PLXx macros are used to access plx registers directly
 * the IPLXx macros are used to access the plx dma registers via ihcp space -
 * which is necessary with the early revs of the plx chip - we could use IHCPx macros
 * to do this, but IPLX should be more readable, and it will be easier to find them
 * to mod later if necessary for some future rev of the chip
 * 
 * the configuration accesses will be done explicitly, without macros
 * 
 * unit_p->xxx_base is defined as a pointer to a 32 bit value, offsets are byte offsets, so /4
 * we could also do things with fancy casts
 * 
 * offset to IOPB_XXXX will be iopb # * IOPB_SIZE + byte offset within the iopb so also /4
 *
 * the previous driver version used direct pointer de-referencing (OK on x86)
 * we leave the pointer math as it was rather than try to cast the bases as u_ints or ?
 * to try to avoid future 64 bit pointer issues
 * 
 */

#define	PLX_GETL(offset)	readl(unit_p->plx_base_p + offset)
#define	PLX_PUTL(offset,value)	writel(value, unit_p->plx_base_p + offset) 
#define	IPLX_GETL(offset)	readl(unit_p->ihcp_base_p + offset)
#define	IPLX_PUTL(offset,value)	writel(value, unit_p->ihcp_base_p + offset)
#define	IHCP_GETL(offset)	readl(unit_p->ihcp_base_p + offset)
#define	IHCP_PUTL(offset,value)	writel(value, unit_p->ihcp_base_p + offset)
#define	IOPB_GETL(offset)	readl(unit_p->iopb_base_p + offset)
#define	IOPB_PUTL(offset,value)	writel(value, unit_p->iopb_base_p + offset)

/*
 * define size of i/o parameter block (link of dma chain list)
 * and the offsets of elements of the block from the base address
 */

#define IOPB_SIZE               0x10	/* 16 bytes per block   */

#define IOPB_PCI_ADDRESS        0x00	/* starting pci address of user buffer block    */
#define IOPB_LOCAL_ADDRESS      0x04	/* starting local address       */
#define IOPB_TRANSFER_SIZE      0x08	/* number of bytes in this block        */
#define IOPB_NEXT_IOPB          0x0c	/* physical address of next iopb in list        */
